function toMs(minutes) {
  return minutes * 60 * 1000;
} 

function loadTimeouts() {
  Timeout_retrieve(Timeout_parseData,Timeout_connectionProblem);
}


var timeouts = {
  'fcst':{"value":120,"id":null,"lastUpdate":0},
  'PWS':{"value":5,"id":null,"lastUpdate":0},
  'ICAO':{"value":20,"id":null,"lastUpdate":0},
  'alert':{"value":5,"id":null,"lastUpdate":0},
  'self':{"value":180,"id":null,"lastUpdate":0}
};

var Timeout_state = 'running';
var Timeout_pausedAt = now();

Timeout_TIMEOUT = 6000;

function Timeout_retrieve(callback, error_loading) {
  //add t={num} to URL so we force refresh  
  var url = "http://api.wunderground.com/auto/wui/geo/Timeouts/index.xml?t=" + new Date().getTime();
  var hasBeenLoaded = false;
  
  Timeout_state = 'running';
  Timeout_pausedAt = now();

  var request = newXMLHttpRequest();
  if (!request) {if (error_loading) {error_loading();} return;}
  request.onreadystatechange = onReadyStateChange;
  if (request.overrideMimeType) request.overrideMimeType('application/json');
  request.open("GET", url, true);
  request.setRequestHeader("Cache-Control", "no-cache");
  
  var requestTimeout = view.setTimeout(function() { request.abort(); if (error_loading) {error_loading();}}, Timeout_TIMEOUT);   
  
  request.send();

  function onReadyStateChange() {
    if (request.readyState != 4 || hasBeenLoaded == true) {
      return;
    }
    hasBeenLoaded = true;
    view.clearTimeout(requestTimeout);
    if (request.status == 200) {
      debug.trace("Timeout data loaded");
      callback(request.responseText, Timeout_setTimeouts, error_loading);
    } else {
      connection.hasProblem();
      if (error_loading) {error_loading();} 
    }
  }
}

function Timeout_parseData(responseText, callback, error_loading) {
  var receivedTimeouts;
  
  try {
    //should load data the same as the default values above
    eval('receivedTimeouts = ' + responseText);
    
    if (receivedTimeouts.vistaLatestVersion > Gadget_VERSION) {
      Gadget_setUpgradeDialog(receivedTimeouts.vistaLatestVersion,receivedTimeouts.vistaLatestVersionLink);
    } else {
      Gadget_upgradeAlert = null;
    }
    
    if (callback) callback(receivedTimeouts);
    
  } catch(e) {
    debug.error("Timeout_parseData: eval failed. \n"+e);
    if (error_loading) error_loading(callback);
  } 
}
function Timeout_connectionProblem (callback) {
  debug.error("Couldn't receive connection");
  
  receivedTimeouts = {
      'forecast': 120,
      'PWS'     : 5,
      'ICAO'    : 20,
      'photos'  : 60,
      'radar'   : 5,
      'webcam'  : 2,
      'alert'  : 5,
      'self' : 180
  };
  if (callback) callback(receivedTimeouts);
}

function Timeout_setTimeouts (receivedTimeouts) {
  //to maintain backwards compat. w/ our timeout feed
  receivedTimeouts.self = receivedTimeouts.selfTimeout;
  receivedTimeouts.fcst = receivedTimeouts.forecast;

  for (var t in timeouts) {
    if (t in Object.prototype) continue;
    var tObj = timeouts[t];
    if (tObj.value != receivedTimeouts[t] || !tObj.id) {//changed since our last update or first time
      tObj.value = receivedTimeouts[t];
      if (tObj.id) {view.clearTimeout(tObj.id);}
      tObj.id = view.setTimeout("reloadData('"+t+"')",tObj.value * 60 * 1000);
    }    
  }
}

function Timeout_clearTimeouts () {
  for (var t in timeouts) {
    if (t in Object.prototype) continue;
    if (timeouts[t].id) view.clearTimeout(timeouts[t].id);
  }
}

function Timeout_pauseTimeouts() {
  Timeout_clearTimeouts();
  Timeout_state = 'paused';
  Timeout_pausedAt = now(); 
}

function Timeout_resumeTimeouts() {
  var remaining;
  var curTime = now();
  var t; var tObj; var elapsed; var updateMin;
  
  //catch the first time, when we're still visible
  if (Timeout_state == 'running') return;
  Timeout_state = 'running';

  //these execute on every onShow
  var runNow = ["ICAO", "PWS", "alert"];
  for (var i = 0; i < runNow.length; i++)
    reloadData(runNow[i]);
  
  //wait on these to see if we should reload
  var waitAndSee = ['self','fcst'];
  
  for (var i = 0; i < waitAndSee.length; i++) {
    t = waitAndSee[i]; tObj = timeouts[t]; elapsed = curTime - tObj.lastUpdate; updateMin = tObj.value*60;
    if (elapsed >= updateMin) reloadData(t);
    else {
      remaining = updateMin - elapsed;
      tObj.id = view.setTimeout('reloadData("'+t+'")',remaining * 1000);
    }
  }
}

//reloads a given type of data
function reloadData( type ) {
  var tObj = timeouts[type]; if (!tObj) return false;
  tObj.id = view.setTimeout("reloadData('"+type+"');",tObj.value * 60 * 1000);
  
  debug.trace("reloadData('"+type+"')");
  
  switch(type) {
    case "self": loadTimeouts(); break;
    case "fcst": Forecast_retrieve(setForecast, Forecast_errorLoading); break;
    case "PWS":
      var pws = prefs.get("PWS");
      if (pws != null && pws != "") {
        CurCond_retrieve('pws', false, setCurConds, CurCond_errorLoading);
      }
    break;
    case "ICAO":
      var icao = prefs.get("airport"); var pws = prefs.get("PWS");
      if (icao != null && icao != "") {
          var no_overwrite = (pws != null && pws != "") ? true : false;
        debug.trace("no_overwrite = ["+(no_overwrite ? "true":"false")+"]");
        CurCond_retrieve('airport', no_overwrite, setCurConds, CurCond_errorLoading);
      }    
    break;
    case "alert": Alert_retrieve(setAlert,Alert_errorLoading); break;
  }
  
  tObj.lastUpdate = now();
}

//All AJAX functions should check if we are connected; if not,
//we simply poll until we are connected
function Connection() {
  var self = this;
  
  this.isGood = true; //true if connection is bad
  this.pollingForService = null; //boolean if currently polling
  this.pollingInterval = 1 * 60 * 1000; //1 minute
  
  this.hasProblem = function () {
    //MODIFIES: isGood, either T/F
    //EFFECTS: called if there is a connection problem
    //sets off connection polling, freezes all requests
    
    //If we already know we have a problem, just continue
    if (!connection.isGood) return;
    
    self.isGood = false;
    
    Timeout_pauseTimeouts();
    
    self.showConnectionProblem();
  };
  
  this.hasNoProblem = function() {
    //MODIFIES: isGood, either T
    //EFFECTS:  should be called when connection in known to be good
    //          cancels out any polling and resumes all timeouts
    
    self.hideConnectionProblem();
  };
  
  this.testConnection = function() {
    var url = "http://api.wunderground.com/auto/wui/geo/Ping/index.xml?t=" + new Date().getTime();
    var hasBeenLoaded = false;
    var request = newXMLHttpRequest();
    request.onreadystatechange = onReadyStateChange;
    if (request.overrideMimeType) request.overrideMimeType('application/json');
    request.open("GET", url, true);
    request.setRequestHeader("Cache-Control", "no-cache");

    var requestTimeout = view.setTimeout(function() { request.abort(); }, Timeout_TIMEOUT);   

    request.send();

    function onReadyStateChange() {
      if (request.readyState != 4 || hasBeenLoaded == true) {
        return;
      }
      hasBeenLoaded = true;
      view.clearTimeout(requestTimeout);
      if (request.status == 200) {
        self.hideConnectionProblem();
      }
    }
  };
  
  this.showConnectionProblem = function() {
    document.getElementById('DockedTop').style.visibility = "hidden";
    document.getElementById('DockedMiddle_forecast').style.visibility = "hidden";
    document.getElementById('DockedBottom_alertBackground').style.visibility = "hidden";
    document.getElementById('DockedBottom_alertMessage').style.visibility = "hidden";
    
    document.getElementById('connectionProblemLabel').style.visibility = "visible"; //display our connection problem text
    
    Gadget_changeBackgroundHeaderWithCondition('unknown');
    
    self.beginPollingForService();
  };
  
  this.hideConnectionProblem = function() {
    document.getElementById('DockedTop').style.visibility = "visible";
    document.getElementById('DockedMiddle_forecast').style.visibility = "visible";
    document.getElementById('DockedBottom_alertBackground').style.visibility = "visible";
    document.getElementById('DockedBottom_alertMessage').style.visibility = "visible";
    
    document.getElementById('connectionProblemLabel').style.visibility = "hidden"; //display our connection problem text
    
    self.endPollingForService();
    self.isGood = true;
  };
  
  this.beginPollingForService = function() {
    self.pollingForService = setInterval(self.testConnection, self.pollingInterval);
  };
  
  this.endPollingForService = function() {
    
    clearInterval( self.pollingForService );
    self.pollingForService = null;
    
    
    if (localization.isGood === false) {
      localization.getLANG();
    }
    
    Timeout_resumeTimeouts();
  };
}

connection = new Connection();